@@ -2,13 +2,12 @@
 # -*- coding: utf-8 -*-
 
 from datetime import datetime
-from uuid import UUID
+from uuid import UUID, uuid4
 
-import logging
 from tornado import gen
-from motorengine import Q
+from ujson import loads, dumps
 
-from holmes.models import Review
+from holmes.models import Review, Page
 from holmes.handlers import BaseHandler
 
 
@@ -47,41 +46,44 @@ class ReviewHandler(BaseReviewHandler):
         self.write_json(result)
         self.finish()
 
+    def post(self, page_uuid, review_uuid=None):
+        page = Page.by_uuid(page_uuid, self.db)
 
-class CompleteReviewHandler(BaseReviewHandler):
-    @gen.coroutine
-    def post(self, page_uuid, review_uuid):
-        review = None
-        if self._parse_uuid(review_uuid):
-            review = Review.by_uuid(review_uuid, self.db)
+        review_data = loads(self.get_argument('review'))
 
-        if not review:
-            self.set_status(404, 'Review with uuid of %s not found!' % review_uuid)
-            logging.debug('Review with uuid of %s not found!' % review_uuid)
-            self.finish()
-            return
-
-        if review.is_complete:
-            self.set_status(400, 'Review with uuid %s is already completed!' % review_uuid)
-            logging.debug('Review with uuid %s is already completed!' % review_uuid)
-            self.finish()
-            return
-
-        review.is_complete = True
-        review.is_active = True
-        review.completed_date = datetime.now()
+        review = Review(
+            domain=page.domain,
+            page=page,
+            is_active=True,
+            is_complete=False,
+            completed_date=datetime.now(),
+            uuid=uuid4(),
+        )
 
+        self.db.add(review)
         self.db.flush()
 
-        review.page.last_review = review
-        review.page.last_review_date = review.completed_date
+        for fact in review_data['facts']:
+            review.add_fact(fact['key'], fact['value'], fact['title'], fact['unit'])
+        self.db.flush()
+
+        for violation in review_data['violations']:
+            review.add_violation(violation['key'], violation['title'], violation['description'], violation['points'])
+        self.db.flush()
+
+        review.is_complete = True
+        self.db.flush()
+
+        page.last_review = review
+        page.last_review_date = review.completed_date
 
         self.db.flush()
 
         self._remove_older_reviews_with_same_day(review)
 
         self.db.query(Review).filter(
-            Review.page_id == review.page_id,
+            Review.page_id == review.page_id
+        ).filter(
             Review.id != review.id
         ).update({
             'is_active': False
@@ -89,8 +91,10 @@ class CompleteReviewHandler(BaseReviewHandler):
 
         self.db.flush()
 
-        self.write('OK')
-        self.finish()
+        self.application.event_bus.publish(dumps({
+            'type': 'new-review',
+            'reviewId': str(review.uuid)
+        }))
 
     def _remove_older_reviews_with_same_day(self, review):
         dt = datetime.now()
