@@ -4,7 +4,7 @@
 from datetime import datetime, timedelta
 
 from tornado import gen
-from uuid import UUID
+from ujson import dumps
 
 from holmes.models.worker import Worker
 from holmes.handlers import BaseHandler
@@ -12,9 +12,7 @@ from holmes.handlers import BaseHandler
 
 class WorkerHandler(BaseHandler):
 
-    def post(self, uuid, i_am='alive'):
-        worker_uuid = UUID(uuid)
-
+    def post(self, worker_uuid, i_am='alive'):
         worker = Worker.by_uuid(worker_uuid, self.db)
 
         if worker:
@@ -23,15 +21,22 @@ class WorkerHandler(BaseHandler):
             elif i_am == 'dead':
                 self.db.delete(worker)
         else:
-            self.db.add(Worker(uuid=worker_uuid))
+            worker = Worker(uuid=worker_uuid)
+            self.db.add(worker)
+
+        self.db.flush()
 
         self._remove_zombies_workers()
         self.db.flush()
 
+        self.application.event_bus.publish(dumps({
+            'type': 'worker-status',
+            'workerId': str(worker.uuid)
+        }))
+
         self.write(str(worker_uuid))
         self.finish()
 
-    @gen.coroutine
     def _remove_zombies_workers(self):
         dt = datetime.now() - timedelta(seconds=self.application.config.ZOMBIE_WORKER_TIME)
         self.db.query(Worker).filter(Worker.last_ping < dt).delete()
@@ -43,16 +48,6 @@ class WorkersHandler(BaseHandler):
     def get(self):
         workers = self.db.query(Worker).all()
 
-        workers_json = []
-        for worker in workers:
-            worker_dict = worker.to_dict()
-
-            if worker.working:
-                page = worker.current_review.page
-                if page:
-                    worker_dict['page_url'] = page.url
-                    worker_dict['page_uuid'] = str(page.uuid)
-            workers_json.append(worker_dict)
-
+        workers_json = [worker.to_dict() for worker in workers]
         self.write_json(workers_json)
         self.finish()
