@@ -4,30 +4,47 @@
 import sys
 import logging
 from uuid import uuid4
-from os.path import abspath, dirname, join
+from os.path import join
 
 import requests
 from ujson import dumps, loads
 from requests.exceptions import ConnectionError
 from sheep import Shepherd
 from colorama import Fore, Style
+from octopus import TornadoOctopus
 
 from holmes import __version__
 from holmes.config import Config
 from holmes.reviewer import Reviewer, InvalidReviewError
+from holmes.utils import load_classes
 
 
 class HolmesWorker(Shepherd):
     def initialize(self):
-        self.root_path = abspath(join(dirname(__file__), '..'))
-        self.uuid = None
+        self.uuid = uuid4().hex
         self.working = True
 
+        self.facters = self._load_facters()
         self.validators = self._load_validators()
 
-    def post(self, url, data={}):
-        url = join(self.config.HOLMES_API_URL.rstrip('/'), url.lstrip('/'))
+        logging.debug('Starting Octopus with %d concurrent threads.' % self.options.concurrency)
+        self.otto = TornadoOctopus(
+            concurrency=self.options.concurrency, cache=True,
+            connect_timeout_in_seconds=self.config.CONNECT_TIMEOUT_IN_SECONDS,
+            request_timeout_in_seconds=self.config.REQUEST_TIMEOUT_IN_SECONDS
+        )
+        self.otto.start()
 
+    def config_parser(self, parser):
+        parser.add_argument(
+            '--concurrency',
+            '-t',
+            type=int,
+            help='Number of threads to use for Octopus (doing GETs concurrently)'
+        )
+
+    @property
+    def proxies(self):
         proxies = None
         if self.config.HTTP_PROXY_HOST is not None:
             proxy = "%s:%s" % (self.config.HTTP_PROXY_HOST, self.config.HTTP_PROXY_PORT)
@@ -39,7 +56,22 @@ class HolmesWorker(Shepherd):
                 "https": https_proxy,
             }
 
-        return requests.post(url, data=data, proxies=proxies)
+        return proxies
+
+    def async_get(self, url, handler, method='GET', **kw):
+        if self.proxies:
+            kw['proxies'] = self.proxies
+
+        logging.debug('Enqueueing %s for %s...' % (method, url))
+        self.otto.enqueue(url, handler, method, **kw)
+
+    def get(self, url):
+        url = join(self.config.HOLMES_API_URL.rstrip('/'), url.lstrip('/'))
+        return requests.get(url, proxies=self.proxies)
+
+    def post(self, url, data={}):
+        url = join(self.config.HOLMES_API_URL.rstrip('/'), url.lstrip('/'))
+        return requests.post(url, data=data, proxies=self.proxies)
 
     def get_description(self):
         return "%s%sholmes-worker%s (holmes-api v%s)" % (
@@ -60,22 +92,19 @@ class HolmesWorker(Shepherd):
         self.working = False
 
     def do_work(self):
-        if self.uuid is None:
-            self.uuid = uuid4().hex
-
         if self._ping_api():
             err = None
             job = self._load_next_job()
-            if job:
-                self._start_job(job['review'])
-
+            if job and self._start_job(job['url']):
                 try:
                     self._start_reviewer(job=job)
                 except InvalidReviewError:
                     err = str(sys.exc_info()[1])
                     logging.error("Fail to review %s: %s" % (job['url'], err))
 
-                self._complete_job(job['review'], error=err)
+                self._complete_job(error=err)
+            elif job:
+                logging.debug('Could not start job for url "%s". Maybe other worker doing it?' % job['url'])
 
     def _start_reviewer(self, job):
         if job:
@@ -84,16 +113,19 @@ class HolmesWorker(Shepherd):
                 api_url=self.config.HOLMES_API_URL,
                 page_uuid=job['page'],
                 page_url=job['url'],
-                review_uuid=job['review'],
                 config=self.config,
-                validators=self.validators
-                )
+                validators=self.validators,
+                facters=self.facters,
+                async_get=self.async_get,
+                wait=self.otto.wait,
+                wait_timeout=0  # max time to wait for all requests to finish
+            )
 
             reviewer.review()
 
     def _ping_api(self):
         try:
-            self.post('/worker/%s/alive' % self.uuid, data={'worker_uuid': self.uuid})
+            self.post('/worker/%s/alive' % self.uuid)
             return True
         except ConnectionError:
             logging.fatal('Fail to ping API [%s]. Stopping Worker.' % self.config.HOLMES_API_URL)
@@ -102,59 +134,47 @@ class HolmesWorker(Shepherd):
 
     def _load_next_job(self):
         try:
-            response = self.post('/next')
+            response = self.get('/next')
             if response and response.text:
                 return loads(response.text)
         except ConnectionError:
             logging.fatal('Fail to get next review from [%s]. Stopping Worker.' % self.config.HOLMES_API_URL)
             self.stop_work()
 
-    def _start_job(self, review_uuid):
-        if not review_uuid:
+        return None
+
+    def _start_job(self, url):
+        if not url:
             return False
 
         try:
-            response = self.post('/worker/%s/review/%s/start' % (self.uuid, review_uuid))
-            return ('OK' == response.text)
+            response = self.post('/worker/%s/start' % self.uuid, data=url)
+            if response.text == 'OK':
+                return True
 
+            if response.text == 'NOK':
+                logging.debug('Failed to acquire lock on %s.' % url)
+                return False
         except ConnectionError:
             logging.error('Fail to start review.')
 
-    def _complete_job(self, review_uuid, error=None):
-        if not review_uuid:
-            return False
+        return False
 
+    def _complete_job(self, error=None):
         try:
-            url = '/worker/%s/review/%s/complete' % (self.uuid, review_uuid)
+            url = '/worker/%s/complete' % self.uuid
             response = self.post(url, data=dumps({'error': error}))
             return ('OK' == response.text)
         except ConnectionError:
             logging.error('Fail to complete worker.')
 
-    def _load_validators(self, validators=None, validators_to_load=None):
-        if validators_to_load is None:
-            validators_to_load = self.config.VALIDATORS
+        return False
 
-        if validators is None:
-            validators = []
+    def _load_validators(self):
+        return load_classes(default=self.config.VALIDATORS)
 
-        for validator_full_name in validators_to_load:
-            if isinstance(validator_full_name, (tuple, set, list)):
-                self._load_validators(validators, validator_full_name)
-                continue
-
-            try:
-                module_name, class_name = validator_full_name.rsplit('.', 1)
-                module = __import__(module_name, globals(), locals(), class_name)
-                validators.append(getattr(module, class_name))
-            except ValueError:
-                logging.warn('Invalid validator name [%s]. Will be ignored.' % validator_full_name)
-            except AttributeError:
-                logging.warn('Validator [%s] not found. Will be ignored.' % validator_full_name)
-            except ImportError:
-                logging.warn('Module [%s] not found. Will be ignored.' % validator_full_name)
-
-        return validators
+    def _load_facters(self):
+        return load_classes(default=self.config.FACTERS)
 
 
 def main():
